{ **************************************************************************** }
{ *                                                                          * }
{ * SNOOPY TENNIS (Nintendo, Game & Watch Wide Screen) Simulator             * }
{ * by Luca "MADrigal" Antignano, June 2000 (revision: August 2021)          * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit unit1;
{ this unit contains the main form implementation                              }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, Registry, Jpeg, Pngimage, Fmod, Fmodtypes, StdCtrls, Menus,
  ShellAPI;

type
  TForm1 = class(TForm)
{ * HANDHELD GRAPHICS, INCLUDING BUTTONS                                     * }

    // fullscreen mode items
    btn_game_menu_top:        TImage;
    btn_game_menu_background: TImage;

    im_backdrop:              TImage;
    im_backdrop_logo:         TImage;

    // electronic game
    im_background:            TImage;       im_background_screen_top: TImage;   {1-2}

    btn_game_a_down:          TImage;       btn_game_a_top:           TImage;   {3-4}
    btn_game_b_down:          TImage;       btn_game_b_top:           TImage;   {5-6}
    btn_time_down:            TImage;       btn_time_top:             TImage;   {7-8}
    btn_acl_down:             TImage;       btn_acl_top:              TImage;   {9-10}
                                            btn_alarm_top:            TImage;   {11}

    btn_moveup_down:          TImage;       btn_moveup_top:           TImage;   {12-13}
    btn_movedown_down:        TImage;       btn_movedown_top:         TImage;   {14-15}
    btn_hit_top:              TImage;       btn_hit_down:             TImage;   {16-17}

{ * LCD SCREEN GRAPHICS (SPRITES)                                            * }

    // on-screen panel (alarm, time/score, game a/b, etc.)
    im_panel_time_am: TImage; im_panel_time_pm: TImage; im_panel_time_colon: TImage; {18-20}

    im_panel_digit_pos_1: TImage; im_panel_digit_pos_2: TImage; im_panel_digit_pos_3: TImage; im_panel_digit_pos_4: TImage; {21-24}

    im_panel_alarm_1: TImage; im_panel_alarm_2: TImage; {25-26}

    im_panel_miss_1: TImage; im_panel_miss_2: TImage; im_panel_miss_3: TImage; {27-29}

    im_panel_game_a: TImage; im_panel_game_b: TImage; {30-31}

    im_panel_number_0: TImage; im_panel_number_1: TImage; im_panel_number_2: TImage; im_panel_number_3: TImage; im_panel_number_4: TImage;
    im_panel_number_5: TImage; im_panel_number_6: TImage; im_panel_number_7: TImage; im_panel_number_8: TImage; im_panel_number_9: TImage; {32-41}

    // on-screen items
    im_snoopy_1_1: TImage; im_snoopy_1_2: TImage; im_snoopy_1_3: TImage; im_snoopy_1_4: TImage; {42-45}
    im_snoopy_2_1: TImage; im_snoopy_2_2: TImage; im_snoopy_2_3: TImage; im_snoopy_2_4: TImage; {46-49}
    im_snoopy_3_1: TImage; im_snoopy_3_2: TImage; im_snoopy_3_3: TImage; im_snoopy_3_4: TImage; {50-53}

    im_charlie_1: TImage; im_charlie_2: TImage; im_charlie_3: TImage; im_charlie_4: TImage; {54-57}

    im_lucy_0_body: TImage; im_lucy_1_arm_top: TImage; im_lucy_2_arm_middle: TImage; im_lucy_3_arm_bottom: TImage; {58-61}

    im_miss_1_charlie_body: TImage; im_miss_2_charlie_hands_down: TImage; im_miss_3_charlie_hands_up: TImage; im_miss_4_crash: TImage; im_miss_5_snoopy: TImage; im_miss_6_snore_1: TImage; im_miss_7_snore_2: TImage; im_miss_8_snore_3: TImage; {62-69}

    im_ball_1_0: TImage; im_ball_1_1: TImage; im_ball_1_2: TImage; im_ball_1_3: TImage; im_ball_1_4: TImage; im_ball_1_5: TImage; im_ball_1_6: TImage; im_ball_1_7: TImage; im_ball_1_8: TImage; {70-78}
    im_ball_2_0: TImage; im_ball_2_1: TImage; im_ball_2_2: TImage; im_ball_2_3: TImage; im_ball_2_4: TImage; im_ball_2_5: TImage; im_ball_2_6: TImage; im_ball_2_7: TImage; {79-86}
    im_ball_3_0: TImage; im_ball_3_1: TImage; im_ball_3_2: TImage; im_ball_3_3: TImage; im_ball_3_4: TImage; im_ball_3_5: TImage; im_ball_3_6: TImage; {87-93}

    im_ball_4_1: TImage; im_ball_4_2: TImage; im_ball_4_3: TImage; im_ball_4_4: TImage; im_ball_4_5: TImage; im_ball_4_6: TImage; {94-99}
    im_ball_5_1: TImage; im_ball_5_2: TImage; im_ball_5_3: TImage; im_ball_5_4: TImage; im_ball_5_5: TImage; im_ball_5_6: TImage; {100-105}
    im_ball_6_1: TImage; im_ball_6_2: TImage; im_ball_6_3: TImage; im_ball_6_4: TImage; im_ball_6_5: TImage; im_ball_6_6: TImage; {106-111}

{ * TIMERS                                                                   * }
    Timer_Chance:          TTimer;
    Timer_Clock:           TTimer;
    Timer_Game:            TTimer;
    Timer_Game_First_Ball: TTimer;
    Timer_Game_Over:       TTimer;
    Timer_Miss:            TTimer;
    Timer_Miss_Sounds:     TTimer;
    Timer_Racket:          TTimer;
    Timer_Score_Blink:     TTimer;
    Timer_Snore:           TTimer;

{ * POPUP MENU                                                               * }
    pm_0_menu:          TPopupMenu;
    pm_1_about:         TMenuItem;
    pm_2_readme:        TMenuItem;
    pm_3_whatsnew:      TMenuItem;
    pm_4_manual:        TMenuItem;
    pm_5_n1:            TMenuItem;
    pm_6_config:        TMenuItem;
    pm_7_keys:          TMenuItem;
    pm_8_hiscore:       TMenuItem;
    pm_9_n2:            TMenuItem;
  {}pm_10_fullscreen:   TMenuItem;
  {}pm_11_splashscreen: TMenuItem;
{}//pm_12_controller:   TMenuItem; (not required)
  {}pm_13_n3:           TMenuItem;
  {}pm_14_exit:         TMenuItem;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate                    (Sender: TObject);
    procedure FormActivate                  (Sender: TObject);
    procedure FormClose                     (Sender: TObject; var Action: TCloseAction);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Show_All                  (b: boolean);

    procedure Vid_Show_All_Ball             (b: boolean);
    procedure Vid_Show_All_Charlie          (b: boolean);
    procedure Vid_Show_All_Lucy             (b: boolean);
    procedure Vid_Show_All_Miss             (b: boolean);
    procedure Vid_Show_All_Snoopy           (b: boolean);

    procedure Vid_Show_Panel_Miss           (var p: integer);
    procedure Vid_Show_Panel_Score          (var p: integer; b: boolean);

    procedure Vid_Show_Snoopy               (var p: integer; b: boolean);

{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
    procedure Snd_Play                      (pfs: PFSoundSample);
    procedure Snd_Silence;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure im_backgroundMouseDown        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure im_backgroundMouseMove        (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_game_a_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_a_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseDown          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseUp            (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_moveup_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_moveup_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_movedown_topMouseDown     (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_movedown_topMouseUp       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_hit_topMouseDown          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_hit_topMouseUp            (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure FormKeyDown                   (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp                     (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
    procedure Inp_game_a_down;              procedure Inp_game_a_up;
    procedure Inp_game_b_down;              procedure Inp_game_b_up;
    procedure Inp_time_down;                procedure Inp_time_up;
    procedure Inp_acl_down;                 procedure Inp_acl_up;

    procedure Inp_moveup_down;              procedure Inp_moveup_up;
    procedure Inp_movedown_down;            procedure Inp_movedown_up;
    procedure Inp_hit_down;                 procedure Inp_hit_up;

{ * PROCEDURES: GAME                                                         * }
    procedure Gam_Stop_All_Timers;

    procedure Gam_Check_Chance_Time;
    procedure Gam_Game_Start;
    procedure Gam_Move_Ball_Charlie         (p: integer);
    procedure Gam_Move_Ball_Lucy            (p: integer);
    procedure Gam_Move_Lucy                 (p: integer);
    procedure Gam_Set_Difficulty;
    procedure Gam_Set_Game_Stage;
    procedure Gam_Set_Mode;

{ * PROCEDURES: TIMERS                                                       * }
    procedure Timer_ChanceTimer             (Sender: TObject);
    procedure Timer_ClockTimer              (Sender: TObject);
    procedure Timer_GameTimer               (Sender: TObject);
    procedure Timer_Game_First_BallTimer    (Sender: TObject);
    procedure Timer_Game_OverTimer          (Sender: TObject);
    procedure Timer_MissTimer               (Sender: TObject);
    procedure Timer_Miss_SoundsTimer        (Sender: TObject);
    procedure Timer_RacketTimer             (Sender: TObject);
    procedure Timer_Score_BlinkTimer        (Sender: TObject);
    procedure Timer_SnoreTimer              (Sender: TObject);

{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
    procedure pm_0_menuPopup                (Sender: TObject);
    procedure pm_1_aboutClick               (Sender: TObject);
    procedure pm_2_readmeClick              (Sender: TObject);
    procedure pm_3_whatsnewClick            (Sender: TObject);
    procedure pm_4_manualClick              (Sender: TObject);
    procedure pm_6_configClick              (Sender: TObject);
    procedure pm_7_keysClick                (Sender: TObject);
    procedure pm_8_hiscoreClick             (Sender: TObject);
  {}procedure pm_10_fullscreenClick         (Sender: TObject);
  {}procedure pm_11_splashscreenClick       (Sender: TObject);
{}//procedure pm_12_controllerClick         (Sender: TObject);
  {}procedure pm_14_exitClick               (Sender: TObject);

{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
    procedure btn_game_menu_topClick        (Sender: TObject);

    procedure la_menu_itemMouseEnter        (Sender: TObject);
    procedure la_menu_itemMouseLeave        (Sender: TObject);
    procedure la_menu_itemMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure Vid_Display_Game_Menu         (b: boolean);
  end;

const
{ introduced for use with HH2 / now incorporated into main source }

  HH2_CONTENT_NAME    = 'Snoopy Tennis (Nintendo, Game & Watch Wide Screen)';
  HH2_CONTENT_AUTHOR  = 'Luca "MADrigal" Antignano';
  HH2_CONTENT_EMAIL   = 'lucantignano@gmail.com';
  HH2_CONTENT_VERSION = '5.00';
  HH2_CONTENT_DATE    = '2017/12/04';
  HH2_CONTENT_WEBSITE = 'www.madrigaldesign.it/sim';

  { constants for all builds }
  
  APP_NAME    = HH2_CONTENT_NAME + ' Simulator';                                // application's name (appears on taskbar and Splash screen)
{}APP_VERSION = 'Version ' + HH2_CONTENT_VERSION + ' * Final *';                // current version (appears on Splash screen)
{}APP_DATE    = 'Released by ' + HH2_CONTENT_AUTHOR + ' on 4th December, 2017'; // release date (appears on Splash screen)

  URL_EMAIL   = 'mailto:' + HH2_CONTENT_EMAIL;                                  // link to the email address
  URL_PAYPAL  = 'https://www.paypal.me/LucaAntignano';                          // link to the Paypal donation page
  URL_WEBSITE = 'http://' + HH2_CONTENT_WEBSITE + '/';                          // link to the website

  REG_KEY     =  'Software\MADrigal\SnoopyTn50';                                // registry key

  HINT_CLICK  = 'click or press';
  HINT_GAME_A = 'to start game A (hold to view highscore)';
  HINT_GAME_B = 'to start game B (hold to view highscore)';
  HINT_TIME   = 'to view current time (hold to view alarm time)';
  HINT_ACL    = 'to reset the game';
  HINT_ALARM  = 'button disabled';

{}MENU_ITEMS = 13;

var
{ * main objects * }
  Form1:                   TForm1;                                              // main form
  bStartup:                boolean;                                             // manages splash screen at startup
  iMouseX, iMouseY:        integer;                                             // cursor coordinates (to drag main form)
  iReg:                    TRegIniFile;                                         // system registry (to save high-scores)

{ * sound management * }
  pfs_back,
  pfs_ball,
  pfs_chance,
  pfs_gameover,
  pfs_hit,
  pfs_miss:                PFSoundSample;                                       // sound samples

  bSound:                  boolean;                                             // sound enabled? (depending on selected driver on splash screen)

{ * arrays of images * }
  aIm_panel_alarm:         array[1..2]       of TImage;                         // alarm panel
  aIm_panel_miss:          array[1..3]       of TImage;                         // miss marks panel
  aIm_panel_digit_pos:     array[1..4]       of TImage;                         // empty sprites to host score/time digits
  aIm_panel_number:        array[0..9]       of TImage;                         // digits for score/time

  aIm_snoopy:              array[1..3, 1..4] of TImage;                         // Snoopy's frames (i=row; j=frame)
  aIm_charlie:             array[1..4]       of TImage;                         // Charlie Brown's frames
  aIm_lucy:                array[0..3]       of TImage;                         // Lucy's frames
  aIm_miss:                array[1..8]       of TImage;                         // miss intermission frames
  aIm_ball_charlie:        array[1..3, 0..8] of TImage;                         // balls between Snoopy and Charlie Brown
  aIm_ball_lucy:           array[1..3, 1..6] of record                          // balls between Snoopy and Lucy
                                                  Pic: TImage;
                                                  Left, Right: boolean          // which direction the ball moves
                                                end;

{ * arrays of mixed types * }
  aBall_lucy:              array[1..3]       of integer;                        // current Lucy's status
  aKeys:                   array[1..7] of record                                // keys management (values)
                                            int:  integer;
                                            word: word;
                                           end;

  aKeyHints:               array[1..44] of string;                              // keys management (hint on buttons)

  aMenuItem:               array[1..MENU_ITEMS] of TMenuItem;                   // popup menu items
  aMenuLabels:             array[1..MENU_ITEMS] of TLabel;                      // on-screen game menu labels

{ * integers and booleans * }
  iMode,                                                                        // 0=ACL, 1=time, etc.
  iScore,                                                                       // 0-9999
  iScoreNextChance,                                                             // 200, 500, 1200, 1500, 2200 ... 9500
  iHiScoreA,                                                                    // 0-9999
  iHiScoreB,                                                                    // 0-9999
  iMiss,

  iSnoopyPosition,                                                              // 1-3
  iScoreBallCharlie,                                                            // 2 or 5
  iScoreBallLucy,                                                               // 3 or 6
  iRandomGenerator                                                              // varies, depending on the current game score
                         : integer;

{}bFullscreenMode,                                                              // true if video mode is fullscreen (false if "shaped game")
  bGameMenuOpen,                                                                // true if the on-screen menu is visible

  bControls,                                                                    // true while player can actually move Snoopy
  bControlsRacket,                                                              // true while player can actually let Snoopy use the racket
  bKeypressed,                                                                  // true while any up/down/control key is pressed

  bChance,
  bDoMiss,                                                                      // passed to Timer_Game; true if a miss has occurred in Gam_Move_Ball_Charlie/_Lucy
  bDoPlayBall,                                                                  // passed to Timer_Game; true if the ball is sent from Charlie Brown to Snoopy
  bDoPlayBack                                                                   // passed to Timer_Game; true if the ball is bouncing back from Lucy to Snoopy
                         : boolean;

  iLoopChance,
  iLoopGame,                                                                    // 1-6
  iLoopGameGenerator,                                                           // must be a global variable
  iLoopGameRandomRow,                                                           // 1-3 (random row for balls thrown by Charlie Brown
  iLoopMiss,
  iLoopScoreBlink,
  iLoopSnore
                         : integer;

implementation

uses
  about, gfxinit, keys, manual, splash;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.FormCreate(Sender: TObject);
{ sprites on form are associated to arrays of images, needed to "index" them   }
{ variables are initialized: system registry, randomize, game mode, etc.       }
  var
    i: integer;
begin
  // initializes application name (for Windows taskbar)
  Application.Title := APP_NAME;

{}// sets game off the screen while graphics initialise
{}Top  := - Screen.Height;
{}Left := - Screen.Width;

  // on-screen panel (alarm, time/score, game a/b, etc.)
  aIm_panel_alarm[1] := im_panel_alarm_1; aIm_panel_alarm[2] := im_panel_alarm_2;

  aIm_panel_miss[1] := im_panel_miss_1; aIm_panel_miss[2] := im_panel_miss_2; aIm_panel_miss[3] := im_panel_miss_3;

  aIm_panel_digit_pos[1] := im_panel_digit_pos_1; aIm_panel_digit_pos[2] := im_panel_digit_pos_2; aIm_panel_digit_pos[3] := im_panel_digit_pos_3; aIm_panel_digit_pos[4] := im_panel_digit_pos_4;

  aIm_panel_number[0] := im_panel_number_0; aIm_panel_number[1] := im_panel_number_1; aIm_panel_number[2] := im_panel_number_2; aIm_panel_number[3] := im_panel_number_3; aIm_panel_number[4] := im_panel_number_4;
  aIm_panel_number[5] := im_panel_number_5; aIm_panel_number[6] := im_panel_number_6; aIm_panel_number[7] := im_panel_number_7; aIm_panel_number[8] := im_panel_number_8; aIm_panel_number[9] := im_panel_number_9;

  // on-screen items
  aIm_snoopy[1, 1] := im_snoopy_1_1; aIm_snoopy[1, 2] := im_snoopy_1_2; aIm_snoopy[1, 3] := im_snoopy_1_3; aIm_snoopy[1, 4] := im_snoopy_1_4;
  aIm_snoopy[2, 1] := im_snoopy_2_1; aIm_snoopy[2, 2] := im_snoopy_2_2; aIm_snoopy[2, 3] := im_snoopy_2_3; aIm_snoopy[2, 4] := im_snoopy_2_4;
  aIm_snoopy[3, 1] := im_snoopy_3_1; aIm_snoopy[3, 2] := im_snoopy_3_2; aIm_snoopy[3, 3] := im_snoopy_3_3; aIm_snoopy[3, 4] := im_snoopy_3_4;

  aIm_charlie[1] := im_charlie_1; aIm_charlie[2] := im_charlie_2; aIm_charlie[3] := im_charlie_3; aIm_charlie[4] := im_charlie_4;

  aIm_lucy[0] := im_lucy_0_body; aIm_lucy[1] := im_lucy_1_arm_top; aIm_lucy[2] := im_lucy_2_arm_middle; aIm_lucy[3] := im_lucy_3_arm_bottom;

  aIm_miss[1] := im_miss_1_charlie_body; aIm_miss[2] := im_miss_2_charlie_hands_down; aIm_miss[3] := im_miss_3_charlie_hands_up; aIm_miss[4] := im_miss_4_crash; aIm_miss[5] := im_miss_5_snoopy; aIm_miss[6] := im_miss_6_snore_1; aIm_miss[7] := im_miss_7_snore_2; aIm_miss[8] := im_miss_8_snore_3;

  aIm_ball_charlie[1, 0] := im_ball_1_0; aIm_ball_charlie[1, 1] := im_ball_1_1; aIm_ball_charlie[1, 2] := im_ball_1_2; aIm_ball_charlie[1, 3] := im_ball_1_3; aIm_ball_charlie[1, 4] := im_ball_1_4; aIm_ball_charlie[1, 5] := im_ball_1_5; aIm_ball_charlie[1, 6] := im_ball_1_6; aIm_ball_charlie[1, 7] := im_ball_1_7; aIm_ball_charlie[1, 8] := im_ball_1_8;
  aIm_ball_charlie[2, 0] := im_ball_2_0; aIm_ball_charlie[2, 1] := im_ball_2_1; aIm_ball_charlie[2, 2] := im_ball_2_2; aIm_ball_charlie[2, 3] := im_ball_2_3; aIm_ball_charlie[2, 4] := im_ball_2_4; aIm_ball_charlie[2, 5] := im_ball_2_5; aIm_ball_charlie[2, 6] := im_ball_2_6; aIm_ball_charlie[2, 7] := im_ball_2_7;
  aIm_ball_charlie[3, 0] := im_ball_3_0; aIm_ball_charlie[3, 1] := im_ball_3_1; aIm_ball_charlie[3, 2] := im_ball_3_2; aIm_ball_charlie[3, 3] := im_ball_3_3; aIm_ball_charlie[3, 4] := im_ball_3_4; aIm_ball_charlie[3, 5] := im_ball_3_5; aIm_ball_charlie[3, 6] := im_ball_3_6;

  aIm_ball_lucy[3, 1].Pic := im_ball_4_1; aIm_ball_lucy[3, 2].Pic := im_ball_4_2; aIm_ball_lucy[3, 3].Pic := im_ball_4_3; aIm_ball_lucy[3, 4].Pic := im_ball_4_4; aIm_ball_lucy[3, 5].Pic := im_ball_4_5; aIm_ball_lucy[3, 6].Pic := im_ball_4_6;
  aIm_ball_lucy[2, 1].Pic := im_ball_5_1; aIm_ball_lucy[2, 2].Pic := im_ball_5_2; aIm_ball_lucy[2, 3].Pic := im_ball_5_3; aIm_ball_lucy[2, 4].Pic := im_ball_5_4; aIm_ball_lucy[2, 5].Pic := im_ball_5_5; aIm_ball_lucy[2, 6].Pic := im_ball_5_6;
  aIm_ball_lucy[1, 1].Pic := im_ball_6_1; aIm_ball_lucy[1, 2].Pic := im_ball_6_2; aIm_ball_lucy[1, 3].Pic := im_ball_6_3; aIm_ball_lucy[1, 4].Pic := im_ball_6_4; aIm_ball_lucy[1, 5].Pic := im_ball_6_5; aIm_ball_lucy[1, 6].Pic := im_ball_6_6;

  // registry keys management
  iReg := TregIniFile.Create(REG_KEY);                                          // system registry init

{}// init graphics
{}bFullscreenMode := iReg.ReadBool('', 'FullscreenMode', true);
{}pm_10_fullscreen.Checked := bFullscreenMode;
{}Gfxinit.Gfx_Game_Init;
{}bGameMenuOpen := false;

{}// initialises the "hints" array
  for i := 1 to 10 do
    aKeyHints[i] := '''' + Chr(i + 47) + '''';

  for i := 11 to 36 do
    aKeyHints[i] := '''' + Chr(i + 54) + '''';

  aKeyHints[37] := '''up arrow'' key';
  aKeyHints[38] := '''down arrow'' key';
  aKeyHints[39] := '''left arrow'' key';
  aKeyHints[40] := '''right arrow'' key';
  aKeyHints[41] := '''alt'' key';
  aKeyHints[42] := '''ctrl'' key';
  aKeyHints[43] := '''shift'' key';
  aKeyHints[44] := '''spacebar''';

  // reads values from system registry into the inputs array
  aKeys[1].int := iReg.ReadInteger('', 'Key1', 2);                              // "Game A" (default: "1")
  aKeys[2].int := iReg.ReadInteger('', 'Key2', 3);                              // "Game B" (default: "2")
  aKeys[3].int := iReg.ReadInteger('', 'Key3', 4);                              // "Time"   (default: "3")
  aKeys[4].int := iReg.ReadInteger('', 'Key4', 6);                              // "ACL"    (default: "5")
  aKeys[5].int := iReg.ReadInteger('', 'Key5', 37);                             // "Up"     (default: "UP")
  aKeys[6].int := iReg.ReadInteger('', 'Key6', 38);                             // "Down"   (default: "DOWN")
  aKeys[7].int := iReg.ReadInteger('', 'Key7', 42);                             // "Hit"    (default: "CTRL")

  // writes values into the system registry (in case no pre-existing values
  // stored), then assigns key values to the key-inputs array
  for i := 1 to 7 do
    begin
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      KeysForm.AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  btn_game_a_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_GAME_A; // "Game A"
  btn_game_b_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_GAME_B; // "Game B"
  btn_time_top.Hint     := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int] + ' ' + HINT_TIME;   // "Time"
  btn_acl_top.Hint      := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int] + ' ' + HINT_ACL;    // "ACL"
  btn_alarm_top.Hint    := HINT_ALARM;                                                     // "Alarm"
  btn_moveup_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[5].int];                     // "Up"
  btn_movedown_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[6].int];                     // "Down"
  btn_hit_top.Hint      := HINT_CLICK + ' ' + aKeyHints[aKeys[7].int];                     // "Hit"

  // popup menu array association
  aMenuItem[1]  := pm_1_about;
  aMenuItem[2]  := pm_2_readme;
  aMenuItem[3]  := pm_3_whatsnew;
  aMenuItem[4]  := pm_4_manual;
  aMenuItem[5]  := pm_5_n1;
  aMenuItem[6]  := pm_6_config;
  aMenuItem[7]  := pm_7_keys;
  aMenuItem[8]  := pm_8_hiscore;
  aMenuItem[9]  := pm_9_n2;
{}aMenuItem[10] := pm_10_fullscreen;
{}aMenuItem[11] := pm_11_splashscreen;
{}//               pm_12_controller;
{}aMenuItem[12] := pm_13_n3;
{}aMenuItem[13] := pm_14_exit;

  // variables init
  DoubleBuffered := true;                                                       // enables double buffering for rendering of "form1"

  iHiScoreA := iReg.ReadInteger('', 'HiScore1', iHiScoreA);                     // reads value from system registry into iHiScoreA
  iHiScoreB := iReg.ReadInteger('', 'HiScore2', iHiScoreB);                     // reads value from system registry into iHiScoreB

  bControls := false;                                                           // prevents hero movement
  bControlsRacket := false;                                                     // prevents hero movement
  bKeyPressed := false;                                                         // no key is currently being kept pressed

  Randomize;                                                                    // initializes the random number generator
end;





procedure TForm1.FormActivate(Sender: TObject);
{ sends focus to the splash screen at startup                                  }
begin
  if bStartup then
    begin
      bStartup := false;
      SplashForm.Show;
    end;
end;





procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
{ frees memory before shutting down                                            }
begin
  FSOUND_Sample_Free(pfs_back);   FSOUND_Sample_Free(pfs_ball);
  FSOUND_Sample_Free(pfs_chance); FSOUND_Sample_Free(pfs_gameover);
  FSOUND_Sample_Free(pfs_hit);    FSOUND_Sample_Free(pfs_miss);

  FSOUND_Close;

{}iReg.Destroy;
  Action := caFree;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Vid_Show_All(b: boolean);
{ displays/hides all sprites                                                   }
  var
    i: integer;
begin
  for i := 1 to 4 do
    aIm_panel_digit_pos[i].Visible := true;

  if b then
    begin
      aIm_panel_digit_pos[1].Picture := aIm_panel_number[1].Picture;
      aIm_panel_digit_pos[2].Picture := aIm_panel_number[2].Picture;
      aIm_panel_digit_pos[3].Picture := aIm_panel_number[0].Picture;
      aIm_panel_digit_pos[4].Picture := aIm_panel_number[0].Picture;
    end
  else
    for i := 1 to 4 do
      aIm_panel_digit_pos[i].Picture := nil;

  for i := 1 to 2 do
    aIm_panel_alarm[i].Visible := b;

  im_panel_time_am.Visible := b;
  im_panel_time_pm.Visible := false;
  im_panel_time_colon.Visible := b;

  im_panel_game_a.Visible := b;
  im_panel_game_b.Visible := b;

  i := Ord(b) * 3; Vid_Show_Panel_Miss(i);

  Vid_Show_All_Ball(b);
  Vid_Show_All_Charlie(b);
  Vid_Show_All_Lucy(b);
  Vid_Show_All_Miss(b);
  Vid_Show_All_Snoopy(b);
end;





procedure TForm1.Vid_Show_All_Ball(b: boolean);
{ displays/hides all balls at once                                             }
  var
    i, j: integer;
begin
  for i := 0 to 8 do aIm_ball_charlie[1, i].Visible := b;
  for i := 0 to 7 do aIm_ball_charlie[2, i].Visible := b;
  for i := 0 to 6 do aIm_ball_charlie[3, i].Visible := b;

  for j := 1 to 3 do
    for i := 1 to 5 do
      aIm_ball_lucy[j, i].Pic.Visible := b;
end;





procedure TForm1.Vid_Show_All_Charlie(b: boolean);
{ displays/hides all Charlie Brown's sprites at once                           }
  var
    i: integer;
begin
  for i := 1 to 4 do
    aIm_charlie[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Lucy(b: boolean);
{ displays/hides all Lucy's sprites at once                                    }
  var
    i: integer;
begin
  for i := 0 to 3 do
    aIm_lucy[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Miss(b: boolean);
{ displays/hides all the intermission's sprites at once                        }
  var
    i: integer;
begin
  for i := 1 to 8 do
    aIm_miss[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Snoopy(b: boolean);
{ displays/hides all Snoopy's sprites at once                                  }
  var
    i, j: integer;
begin
  for i := 1 to 3 do
    for j := 1 to 4 do
      aIm_snoopy[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_Panel_Miss(var p: integer);
{ displays as many sprites as p on the misses counter (0=nothing visible)      }
  var
    i: integer;
begin
  for i := 1 to 3 do
    aIm_panel_miss[i].Visible := (i <= p);                                      // displays items 1 to p, hides the remaining ones
end;





procedure TForm1.Vid_Show_Panel_Score(var p: integer; b: boolean);
{ transforms score/high-score into sprites, and displays/hides them            }
  var
    i, t: integer;
begin
  t := p;

  while t >= 10000 do                                                           // displays "10000" as "...0"
    Dec(t, 10000);

  if b then                                                                     // display or hide score/high-score?
    begin                                                                         // display
      if t > 999 then aIm_panel_digit_pos[1].Picture := aIm_panel_number[t div 1000 mod 10].Picture else aIm_panel_digit_pos[1].Picture := nil; // thousands
      if t >  99 then aIm_panel_digit_pos[2].Picture := aIm_panel_number[t div  100 mod 10].Picture else aIm_panel_digit_pos[2].Picture := nil; // hundreds
      if t >   9 then aIm_panel_digit_pos[3].Picture := aIm_panel_number[t div   10 mod 10].Picture else aIm_panel_digit_pos[3].Picture := nil; // tens
                      aIm_panel_digit_pos[4].Picture := aIm_panel_number[t          mod 10].Picture;                                            // units
    end
  else                                                                            // hide
    for i := 1 to 4 do
      aIm_panel_digit_pos[i].Picture := nil;
end;





procedure TForm1.Vid_Show_Snoopy(var p: integer; b: boolean);
{ displays snoopy with his racket down; other racket sprites are always hidden }
begin
  aIm_snoopy[p, 1].Visible := b;
  aIm_snoopy[p, 2].Visible := false;
  aIm_snoopy[p, 3].Visible := false;
  aIm_snoopy[p, 4].Visible := b;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Snd_Play(pfs: PFSoundSample);
{ plays a certain sound sample                                                 }
begin
  if bSound then
    FSOUND_PlaySound(FSOUND_FREE, pfs);
end;





procedure TForm1.Snd_Silence;
{ stops any currently playing sound sample                                     }
begin
  if bSound then
    FSOUND_StopSound(FSOUND_ALL);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.im_backgroundMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
begin
{}if (not bFullscreenMode)
    and (Button = mbLeft) then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TForm1.im_backgroundMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
begin
{}if (not bFullscreenMode)
    and (ssLeft in Shift) then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TForm1.btn_game_a_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game A button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_a_down;
end;





procedure TForm1.btn_game_a_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game A button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_a_up;
end;





procedure TForm1.btn_game_b_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game B button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_b_down;
end;





procedure TForm1.btn_game_b_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game B button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_b_up;
end;





procedure TForm1.btn_time_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ time button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_time_down;
end;





procedure TForm1.btn_time_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ time button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_time_up;
end;





procedure TForm1.btn_acl_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: PRESS                                                            }
begin
  if Button = mbLeft then
    Inp_acl_down;
end;





procedure TForm1.btn_acl_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: RELEASE                                                          }
begin
  if Button = mbLeft then
    Inp_acl_up;
end;





procedure TForm1.btn_moveup_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ UP button: PRESS                                                             }
begin
  if Button = mbLeft then
    Inp_moveup_down;
end;





procedure TForm1.btn_moveup_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ UP button: RELEASE                                                           }
begin
  if Button = mbLeft then
    Inp_moveup_up;
end;





procedure TForm1.btn_movedown_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ DOWN button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_movedown_down;
end;





procedure TForm1.btn_movedown_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ DOWN button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_movedown_up;
end;





procedure TForm1.btn_hit_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ HIT button: PRESS                                                            }
begin
  if Button = mbLeft then
    Inp_hit_down;
end;





procedure TForm1.btn_hit_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ HIT button: RELEASE                                                          }
begin
  if Button = mbLeft then
    Inp_hit_up;
end;





procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_down   else                            // Game A
  if Key = aKeys[2].word then Inp_game_b_down   else                            // Game B
  if Key = aKeys[3].word then Inp_time_down     else                            // Time
  if Key = aKeys[4].word then Inp_acl_down      else                            // ACL
  if Key = aKeys[5].word then Inp_moveup_down   else                            // up
  if Key = aKeys[6].word then Inp_movedown_down else                            // down
  if Key = aKeys[7].word then Inp_hit_down;                                     // hit
end;





procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: RELEASE                                                            }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_up   else                              // Game A
  if Key = aKeys[2].word then Inp_game_b_up   else                              // Game B
  if Key = aKeys[3].word then Inp_time_up     else                              // Time
  if Key = aKeys[4].word then Inp_acl_up      else                              // ACL
  if Key = aKeys[5].word then Inp_moveup_up   else                              // up
  if Key = aKeys[6].word then Inp_movedown_up else                              // down
  if Key = aKeys[7].word then Inp_hit_up;                                       // hit
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Inp_game_a_down;
{ game A button: PRESS                                                         }
begin
  btn_game_a_down.Visible := true;

  if (iMode = 1) or                                                             // ACL/2     or
     (iMode = 3) or                                                             // time      or
     (iMode = 8) then                                                           // game over ->
    begin
      iMode := 4; Gam_Set_Mode;                                                   //         -> hiscore A
    end;
end;





procedure TForm1.Inp_game_a_up;
{ game A button: RELEASE                                                       }
begin
  btn_game_a_down.Visible := false;

  if iMode = 4 then                                                             // hiscore A ->
    begin
      iMode := 5; Gam_Set_Mode;                                                   //         -> game A
    end;
end;





procedure TForm1.Inp_game_b_down;
{ game B button: PRESS                                                         }
begin
  btn_game_b_down.Visible := true;

  if (iMode = 1) or                                                             // ACL/2     or
     (iMode = 3) or                                                             // time      or
     (iMode = 8) then                                                           // game over ->
    begin
      iMode := 6; Gam_Set_Mode;                                                   //         -> hiscore B
    end;
end;





procedure TForm1.Inp_game_b_up;
{ game B button: RELEASE                                                       }
begin
  btn_game_b_down.Visible := false;

  if iMode = 6 then                                                             // hiscore B ->
    begin
      iMode := 7; Gam_Set_Mode;                                                   //         -> game B
    end;
end;





procedure TForm1.Inp_time_down;
{ time button: PRESS                                                           }
begin
  btn_time_down.Visible := true;

  if (iMode = 1) or                                                             // ACL/2     or
     (iMode = 3) or                                                             // time      or
     (iMode = 8) then                                                           // game over ->
    begin
      iMode := 2; Gam_Set_Mode;                                                   //         -> alarm
    end;
end;





procedure TForm1.Inp_time_up;
{ time button: RELEASE                                                         }
begin
  btn_time_down.Visible := false;

  if iMode = 2 then                                                             // alarm ->
    begin
      iMode := 3; Gam_Set_Mode;                                                   //     -> time
    end;
end;





procedure TForm1.Inp_acl_down;
{ ACL button: PRESS                                                            }
begin
  btn_acl_down.Visible := true;

  iMode := 0; Gam_Set_Mode;                                                     // any mode -> ACL/1
end;





procedure TForm1.Inp_acl_up;
{ ACL button: RELEASE                                                          }
begin
  btn_acl_down.Visible := false;

  iMode := 1; Gam_Set_Mode;                                                     // ACL/1  -> ACL/2
end;





procedure TForm1.Inp_moveup_down;
{ UP button: PRESS                                                             }
begin
  btn_moveup_down.Visible := true;

  if ( (iMode = 5) or (iMode = 7) ) and bControls and (not bKeyPressed) then
    begin
      bKeyPressed := true;

      if iSnoopyPosition > 1 then
        begin
          bControlsRacket := true;

          Vid_Show_Snoopy(iSnoopyPosition, false);
          Dec(iSnoopyPosition);
          Vid_Show_Snoopy(iSnoopyPosition, true);
        end;
    end;
end;





procedure TForm1.Inp_moveup_up;
{ UP button: RELEASE                                                           }
begin
  btn_moveup_down.Visible := false;

  bKeyPressed := false;
end;





procedure TForm1.Inp_movedown_down;
{ DOWN button: PRESS                                                           }
begin
  btn_movedown_down.Visible := true;

  if ( (iMode = 5) or (iMode = 7) ) and bControls and (not bKeyPressed) then
    begin
      bKeyPressed := true;

      if iSnoopyPosition < 3 then
        begin
          bControlsRacket := true;

          Vid_Show_Snoopy(iSnoopyPosition, false);
          Inc(iSnoopyPosition);
          Vid_Show_Snoopy(iSnoopyPosition, true);
        end;
    end;
end;





procedure TForm1.Inp_movedown_up;
{ DOWN button: RELEASE                                                         }
begin
  btn_movedown_down.Visible := false;

  bKeyPressed := false;
end;





procedure TForm1.Inp_hit_down;
{ HIT button: PRESS                                                            }
begin
  btn_hit_down.Visible := true;

  if ( (iMode = 5) or (iMode = 7) ) and bControlsRacket and (not bKeyPressed) then
    begin
      bKeyPressed := true;
      bControlsRacket := false;

      aIm_snoopy[iSnoopyPosition, 3].Visible := false;
      aIm_snoopy[iSnoopyPosition, 4].Visible := false;
      aIm_snoopy[iSnoopyPosition, 2].Visible := true;

      if aIm_ball_lucy[iSnoopyPosition, 5].Pic.Visible then
        begin
          bControls := true;

          aIm_ball_charlie[iSnoopyPosition, 1].Visible := false;
          aIm_ball_lucy[iSnoopyPosition, 5].Right := false;
          aIm_ball_lucy[iSnoopyPosition, 5].Left := true;

          Inc(iScore, iScoreBallLucy);

          // brings score to a lower value than 10.000, and resets "iScoreNextChance" to 200
          if iScore >= 10000 then
            begin
              Dec(iScore, 10000);
              iScoreNextChance := 200;
            end;

          Vid_Show_Panel_Score(iScore, true);
          Gam_Set_Difficulty;
          Gam_Check_Chance_Time;

          Snd_Silence;
          Snd_Play(pfs_hit);
        end
      else
        if aIm_ball_charlie[iSnoopyPosition, 1].Visible then
          begin
            bControls := true;

            aIm_ball_charlie[iSnoopyPosition, 1].Visible := false;
            aIm_ball_lucy[iSnoopyPosition, 5].Pic.Visible := true;
            aIm_ball_lucy[iSnoopyPosition, 5].Left := true;

            Inc(iScore, iScoreBallCharlie);

            // brings score to a lower value than 10.000, and resets "iScoreNextChance" to 200
            if iScore >= 10000 then
              begin
                Dec(iScore, 10000);
                iScoreNextChance := 200;
              end;

            Vid_Show_Panel_Score(iScore, true);
            Gam_Set_Difficulty;
            Gam_Check_Chance_Time;

            Snd_Silence;
            Snd_Play(pfs_hit);
          end
        else
          bControls := false;

      Timer_Racket.Enabled := false;

      // to evaluate in case the "chance time" intermission hasn't just got started
      if iLoopChance <> 1 then
        begin
          Timer_Racket.Interval := 380;
          Timer_Racket.Enabled := true;
        end;
    end;
end;





procedure TForm1.Inp_hit_up;
{ HIT button: RELEASE                                                          }
begin
  btn_hit_down.Visible := false;

  bKeyPressed := false;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME                                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Gam_Stop_All_Timers;
{ stops all currently enabled timers                                           }
begin
  Timer_Chance.Enabled          := false;
  Timer_Clock.Enabled           := false;
  Timer_Game.Enabled            := false;
  Timer_Game_First_Ball.Enabled := false;
  Timer_Game_Over.Enabled       := false;
  Timer_Miss.Enabled            := false;
  Timer_Miss_Sounds.Enabled     := false;
  Timer_Racket.Enabled          := false;
  Timer_Score_Blink.Enabled     := false;
  Timer_Snore.Enabled           := false;
end;





procedure TForm1.Gam_Check_Chance_Time;
{ stops action, sets the score value for the next "chance time", and executes  }
{ the "chance time" intermission"                                              }
  var
    i: integer;
begin
  if (not bChance) and (iScore >= iScoreNextChance) then
    begin
      // stops action
      Gam_Stop_All_Timers;
      Snd_Silence;
      bControls := false;
      bControlsRacket := false;

      // loops iScoreNextChance to 200, 500, 1200, 1500, ... , 9200, 9500
      i := iScore;

      while i > 1000 do
        Dec(i, 1000);

      if i >= 500 then
        Inc(iScoreNextChance, 700)
      else
        if i >= 200 then
          Inc(iScoreNextChance, 300);

      // executes the "chance time" intermission
      iLoopChance := 1;
      Timer_Chance.Interval := 100;
      Timer_Chance.Enabled := true;
    end;
end;





procedure TForm1.Gam_Game_Start;
{ initializes the new game                                                     }
begin
  iScore := 0; Vid_Show_Panel_Score(iScore, true);
  iScoreNextChance := 200;
  iMiss := 0;

  Gam_Set_Difficulty;
  Timer_Game.Enabled := true;

  bControls := true;
  bControlsRacket := true;
end;





procedure TForm1.Gam_Move_Ball_Charlie(p: integer);
{ balls thrown by Charlie Brown move                                           }
  var
    i: integer;
begin
  // sets sound effect to play
  for i := 1 to (8 - p) do
    if aIm_ball_charlie[p, i].Visible then
      bDoPlayBall := true;

  // sets ball movement and display
  for i := 0 to (8 - p) do
    aIm_ball_charlie[p, i].Visible := aIm_ball_charlie[p, i + 1].Visible;

  aIm_ball_charlie[p, 8 - p].Visible := false;

  // miss check
  if aIm_ball_charlie[p, 0].Visible then
    begin
      aIm_ball_lucy[p, 5].Pic.Visible := false;
      bDoMiss := true;
    end;
end;





procedure TForm1.Gam_Move_Ball_Lucy(p: integer);
{ balls thrown by Lucy move                                                    }
  var
    i: integer;
begin
  // sets sound effect to play
  for i := 1 to 5 do
    if aIm_ball_lucy[p, i].Pic.Visible then
      bDoPlayBall := true;

  // sets ball movement and display
  for i := 5 downto 0 do
    aIm_ball_lucy[p, i + 1].Right := aIm_ball_lucy[p, i].Right;

  for i := 2 to 6 do
    aIm_ball_lucy[p, i - 1].Left := aIm_ball_lucy[p, i].Left;

   aIm_ball_lucy[p, 1].Right := false;
   aIm_ball_lucy[p, 6].Left := false;

  for i := 1 to 6 do
    aIm_ball_lucy[p, i].Pic.Visible := (aIm_ball_lucy[p, i].Left) or (aIm_ball_lucy[p, i].Right);

  // miss check
  if aIm_ball_lucy[p, 6].Pic.Visible then
    begin
      aIm_ball_charlie[p, 1].Visible := false;
      bDoMiss := true;
    end;
end;





procedure TForm1.Gam_Move_Lucy(p: integer);
{ Lucy appears and bounces balls back to Snoopy                                }
  var
    i,
    rnd:   integer;

    bGen,
    bMove: boolean;
begin
  // Lucy appears on screen randomly (generation)
  with aIm_ball_lucy[p, 3] do
    if (Pic.Visible and Left) and (aBall_lucy[p] = 0) then
      begin
        bGen := (Random(2) = 0);

        for i := 1 to 4 do
          if aIm_ball_charlie[p, i].Visible then
            bGen := false;

        if aIm_ball_lucy[p, 4].Pic.Visible or aIm_ball_lucy[p, 5].Pic.Visible then
          bGen := false;

        if Timer_Game_First_Ball.Enabled then
          bGen := false;

        if bGen then aBall_lucy[p] := 1;
      end;

  // Lucy's animation
  case aBall_lucy[p] of
    1: begin
         aIm_lucy[0].Visible := true;

         aBall_lucy[p] := 2;
       end;
    2: begin
         bMove := true;

         for i := 1 to 3 do
           if (p <> i) and (aBall_lucy[i] = 3)
            then bMove := false;

         if bMove then
           for i := 1 to 3 do
             aIm_lucy[i].Visible := (i = p);

         aBall_lucy[p] := 3;
       end;
    3: begin
         for i := 1 to 3 do
           aIm_lucy[i].Visible := (i = p); { displays the right racket (top/middle/bottom) }

         aIm_ball_lucy[p, 1].Left := false;
         aIm_ball_lucy[p, 1].Right := true;
         bDoPlayBack := true;

         aBall_lucy[p] := 4;
       end;
    4: aBall_lucy[p] := 5; { pause }
    5: begin
         aBall_lucy[p] := 0; { ends loop }

         bMove := true;

         // hides Lucy if there are no balls to bounce anymore
         for i := 1 to 3 do
           if aBall_lucy[i] <> 0 then
             bMove := false;

         if bMove then
           begin
             aIm_lucy[0].Visible:=false;
             aIm_lucy[p].Visible:=false;
           end;
        end;
  end; // case..of
end;





procedure TForm1.Gam_Set_Difficulty;
{ calculates difficulty increase, depending on current score                   }
  var
    i, h, t: integer;
begin
  i := iScore;
  t := 0;
  h := 0;
  
  while i >= 1000 do begin Dec(i, 1000); Inc(t); end; // calculates thousands
  while i >= 100  do begin Dec(i,  100); Inc(h); end; // calculates hundreds
  i := i div 10;                                      // calculate tens

  case iMode of
    5: begin
         case i of
           0: iRandomGenerator := 11;
           1: iRandomGenerator := 10;
           2: iRandomGenerator := 9;
        3..5: iRandomGenerator := 8;
        6..8: iRandomGenerator := 7;
           9: iRandomGenerator := 6;
         end;

         Timer_Game.Interval := 220;
       end;
    7: case i of
         0: begin iRandomGenerator := 11; Timer_Game.Interval := 220; end;
         1:                               Timer_Game.Interval := 210;
         2: begin iRandomGenerator :=  9; Timer_Game.Interval := 200; end;
         3:                               Timer_Game.Interval := 190;
         4: begin iRandomGenerator :=  7; Timer_Game.Interval := 180; end;
         5:                               Timer_Game.Interval := 170;
         6: begin iRandomGenerator :=  6; Timer_Game.Interval := 165; end;
         7:                               Timer_Game.Interval := 160;
         8: begin iRandomGenerator :=  5; Timer_Game.Interval := 155; end;
         9: begin iRandomGenerator :=  4; Timer_Game.Interval := 150; end;
       end;
  end; // case..of

  // finalizes the value of "iRandomGenerator"
  Dec(iRandomGenerator, t);
  Dec(iRandomGenerator, h);

  if iRandomGenerator <= 4 then
    iRandomGenerator := 4;
end;





procedure TForm1.Gam_Set_Game_Stage;
{ sets the stage ready for a new game                                          }
  var
    i, j: integer;
begin
  Vid_Show_All_Miss(false);
  Vid_Show_All_Lucy(false);

  // resets Lucy's array
  for i := 1 to 3 do
    aBall_lucy[i] := 0;

  for i := 1 to 3 do
    for j := 1 to 6 do
      begin
        aIm_ball_lucy[i, j].Left := false;
        aIm_ball_lucy[i, j].Right := false;
      end;

  // displays Snoopy and Charlie Brown
  iSnoopyPosition := 2; Vid_Show_Snoopy(iSnoopyPosition, true);

  for i := 1 to 3 do aIm_charlie[i].Visible := true;

  // sets variables for balls score and "chance time"
  iScoreBallCharlie := 2;
  iScoreBallLucy    := 3;
  bChance := false;

  // sets variables for Timer_Game loops, random, etc.
  iLoopGame := 1;
  iLoopGameGenerator := 0;
  iLoopGameRandomRow := 0;
  bDoMiss := false;

  // first ball is set on screen (needed by the balls and Lucy generators)
  if iScore < 15 then { valid at game start and in case of very early miss }
    Timer_Game_First_Ball.Interval := (Timer_Game.Interval * 6 * 22)
  else
    Timer_Game_First_Ball.Interval := (Timer_Game.Interval * 6 * 11);

  Timer_Game_First_Ball.Enabled := true;
end;





procedure TForm1.Gam_Set_Mode;
{ sets game/time/ACL mode, depending on the value of the iMode variable        }
begin
  case iMode of
    0: begin                                                                    // ACL 1 (ACL button press)
         Gam_Stop_All_Timers;
         Snd_Silence;

         bControls := false;
       end;
    1: Vid_Show_All(true);                                                      // ACL 2 (ACL button release)
    2: begin                                                                    // ALARM (Time button press)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);

         aIm_panel_alarm[1].Visible := true;

         im_panel_time_pm.Visible := true;
         im_panel_time_colon.Visible := true;

         aIm_panel_digit_pos[1].Picture := aIm_panel_number[1].Picture;
         aIm_panel_digit_pos[2].Picture := aIm_panel_number[2].Picture;
         aIm_panel_digit_pos[3].Picture := aIm_panel_number[0].Picture;
         aIm_panel_digit_pos[4].Picture := aIm_panel_number[0].Picture;

         bControls := false;
       end;
    3: begin                                                                    // TIME (Time button release)
         Gam_Stop_All_Timers;

         aIm_panel_alarm[1].Visible := false;

         Timer_Clock.Interval := 1;
         Timer_Clock.Enabled := true;

         bControls := false;
       end;
 4, 6: begin                                                                    // HISCORE A/B (Game A button press)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);

         Gam_Set_Game_Stage;

         bControls := false;

         case iMode of
           4: begin
                im_panel_game_a.Visible := true; Vid_Show_Panel_Score(iHiScoreA, true);
              end;
           6: begin
                im_panel_game_b.Visible := true; Vid_Show_Panel_Score(iHiScoreB, true);
              end;
         end; // case..of
       end;
 5, 7: Gam_Game_Start;                                                          // GAME A/B (Game A/B button release)
    8: begin                                                                    // GAME OVER
         // do nothing
       end;
  end; // case..of
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS                                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_ChanceTimer(Sender: TObject);
{ "chance time" intermission                                                   }
  var
    i: integer;
begin
  case iLoopChance of
    1: begin
         Snd_Silence;
         Snd_Play(pfs_chance);

         Timer_Chance.Interval := 250;
       end;
 2..9: begin
         Snd_Play(pfs_chance);

         // blinks if iMiss <> 0, else displays nothing
         i := iMiss * Ord(Odd(iLoopChance));
         Vid_Show_Panel_Miss(i);
       end;
   10: begin
         if iMiss = 0 then
           // "chance time" starts here
           begin
             iScoreBallCharlie := 5;
             iScoreBallLucy    := 6;
             bChance := true;

             iLoopScoreBlink := - Random(25); // adds (0 to 25 * 800 msec = ) 0 to 20 seconds to the standard duration
             Timer_Score_Blink.Interval := 800;
             Timer_Score_Blink.Enabled := true;
           end;

         // resets misses to zero
         iMiss := 0; Vid_Show_Panel_Miss(iMiss);

         // game continues
         Vid_Show_Snoopy(iSnoopyPosition, true);

         Gam_Set_Difficulty;
         Timer_Game.Enabled := true;

         bControls := true;
         bControlsRacket := true;

         iLoopChance := 1; { needed by the racket animation routine }
         Timer_Chance.Enabled := false;
       end;
   end; // case..of

  // loops action
  Inc(iLoopChance);
end;





procedure TForm1.Timer_ClockTimer(Sender: TObject);
{ transforms current time into "sprites" form, then displays it                }
  var
    present:              TDateTime;
    hour, min, sec, msec: Word;
begin
  present := Now;                                                               // gets system time...
  DecodeTime(present, hour, min, sec, msec);                                    // and decodes it

  if hour > 11 then                                                             // changes mode from 0/24 to 12 AM/12 PM
    begin
      Dec(hour, 12);
      im_panel_time_am.Visible := false;
      im_panel_time_pm.Visible := true;
    end
  else
    begin
      im_panel_time_am.Visible := true;
      im_panel_time_pm.Visible := false;
    end;

  if hour = 0 then                                                              // changes mode from 0/24 to 12 AM/12 PM
    hour := 12;

  if hour > 9 then                                                              // hour: tens
    aIm_panel_digit_pos[1].Picture := aIm_panel_number[hour div 10].Picture
  else
    aIm_panel_digit_pos[1].Picture := nil;

  aIm_panel_digit_pos[2].Picture := aIm_panel_number[hour mod 10].Picture;      // hour: units
  aIm_panel_digit_pos[3].Picture := aIm_panel_number[min  div 10].Picture;      // minute: tens
  aIm_panel_digit_pos[4].Picture := aIm_panel_number[min  mod 10].Picture;      // minute: units

  Timer_Clock.Interval := 1000;
end;





procedure TForm1.Timer_GameTimer(Sender: TObject);
{ main game action                                                             }
  var
    i, j,
    iGenerateRow:   integer;

    bBallsOnScreen: boolean;
begin
  bDoPlayBall := false;
  bDoPlayBack := false;

  // balls movement
  iGenerateRow := 0;

  case iLoopGame of
    1: begin iGenerateRow := 1; Gam_Move_Ball_Charlie(1); Gam_Move_Ball_Lucy(1); Gam_Move_Lucy(1); end;
    2: begin                                              Gam_Move_Ball_Lucy(3); Gam_Move_Lucy(3); end;
    3: begin iGenerateRow := 2; Gam_Move_Ball_Charlie(2); Gam_Move_Ball_Lucy(2); Gam_Move_Lucy(2); end;
    4: begin                                              Gam_Move_Ball_Lucy(1); Gam_Move_Lucy(1); end;
    5: begin iGenerateRow := 3; Gam_Move_Ball_Charlie(3); Gam_Move_Ball_Lucy(3); Gam_Move_Lucy(3); end;
    6: begin                                              Gam_Move_Ball_Lucy(2); Gam_Move_Lucy(2); end;
  end; // case..of

  // loops action
  Inc(iLoopGame);

  if iLoopGame = 7 then
    iLoopGame := 1;

  // generates new balls or executes the "miss" intermission
  if not bDoMiss then
    begin
      // checks whether there are balls on the screen
      bBallsOnScreen := false;

      for i := 1 to 7 do if aIm_ball_charlie[1 ,i].Visible then bBallsOnScreen := true;
      for i := 1 to 6 do if aIm_ball_charlie[2, i].Visible then bBallsOnScreen := true;
      for i := 1 to 5 do if aIm_ball_charlie[3, i].Visible then bBallsOnScreen := true;
      for i := 1 to 3 do for j := 1 to 5 do if aIm_ball_lucy[i, j].Pic.Visible then bBallsOnScreen := true;

      // generates ball
      if iGenerateRow <> 0 then
        begin

          if (iLoopGameRandomRow = 0)
            and ( (not bBallsOnScreen)
                   or
                   ((not Timer_Game_First_Ball.Enabled) and (Random(iRandomGenerator) = 0)) ) then
            begin
              iLoopGameRandomRow := Random(3) + 1;
              iLoopGameGenerator := 1
            end;

          case iLoopGameGenerator of
            1: if iGenerateRow = iLoopGameRandomRow then
                 begin
                   aIm_charlie[3].Visible := false;
                   aIm_ball_charlie[iLoopGameRandomRow, 9 - iLoopGameRandomRow].Visible := true;

                   bDoPlayBall := true;
                   iLoopGameGenerator := 2
                 end;
            2: if iGenerateRow = iLoopGameRandomRow then
                 begin
                   aIm_charlie[2].Visible := false;
                   aIm_ball_charlie[iLoopGameRandomRow, 9 - iLoopGameRandomRow].Visible := false;

                   aIm_charlie[4].Visible := true;
                   aIm_ball_charlie[iLoopGameRandomRow, 8 - iLoopGameRandomRow].Visible := true;

                   bDoPlayBack := true;
                   iLoopGameGenerator := 3;
                 end;
            3: iLoopGameGenerator := 4;
            4: begin
                 aIm_charlie[2].Visible := true;
                 aIm_charlie[3].Visible := true;
                 aIm_charlie[4].Visible := false;

                 iLoopGameRandomRow := 0;
               end;
          end; // case..of
        end;

      // sound effect: which one to play?
      if bDoPlayBack then
        Snd_Play(pfs_back)
      else
        if bDoPlayBall then
          Snd_Play(pfs_ball);
    end
  // a miss has occurred!
  else
    begin
      Gam_Stop_All_Timers;
      Snd_Silence;

      Vid_Show_Panel_Score(iScore, true);

      bControls := false;
      bControlsRacket := false;

      Snd_Play(pfs_miss);

      iLoopMiss := 1;
      Timer_Miss.Interval := 200;
      Timer_Miss.Enabled := true;
    end;
end;





procedure TForm1.Timer_Game_First_BallTimer(Sender: TObject);
{ the first ball has been hit, further balls are generated randomly; Lucy is   }
{ unlocked                                                                     }
begin
  Timer_Game_First_Ball.Enabled := false;
end;





procedure TForm1.Timer_Game_OverTimer(Sender: TObject);
{ game goes into "time" mode, five minutes after game is over                  }
begin
  Timer_Snore.Enabled := false; { must be stopped before Vid_Show_All(false) }

  Vid_Show_All(false);

  iMode := 3; Gam_Set_Mode;

  Timer_Clock.Interval := 1;
  Timer_Clock.Enabled := true;

  Timer_Game_Over.Enabled := false;
end;





procedure TForm1.Timer_MissTimer(Sender: TObject);
{ a miss has occured. executes "miss" intermission                             }
  var
    i, j: integer;
begin
  case iLoopMiss of
    1: begin
         // forces score displaying (in case it was blinking in "chance time")
         Vid_Show_Panel_Score(iScore, true);

         Timer_Miss.Interval := 250;
       end;
    2: begin
         // hides all balls
         aIm_charlie[3].Visible := false;
         Vid_Show_All_Ball(false);

         // hides Lucy
         Vid_Show_All_Lucy(false);

         // hides the missed balls (rightmost column)
         for i := 1 to 3 do
           begin
             aIm_ball_charlie[i, 0].Visible := false;
             aIm_ball_lucy[i, 6].Pic.Visible := false;
           end;

         // increases the number of misses
         Inc(iMiss);

         // the sound effect duration varies, depending on how many misses were made
         case iMiss of
           2: Timer_Miss_Sounds.Interval := 850;
           1: Timer_Miss_Sounds.Interval := 950;
           0: Timer_Miss_Sounds.Interval := 1300;
         end; // case..of

         Timer_Miss_Sounds.Enabled := true;

         Timer_Miss.Interval := 750;
       end;
    3: begin
         aIm_miss[4].Visible := true;

         Timer_Miss.Interval := 450;
       end;
    4: begin
         Vid_Show_Snoopy(iSnoopyPosition, false);
         aIm_miss[5].Visible := true;
         Vid_Show_Panel_Miss(iMiss);
       end;
    5: begin
         aIm_miss[6].Visible := true;

         Timer_Miss.Interval := 350;
       end;
    6: begin
         aIm_miss[1].Visible := true;
         aIm_miss[2].Visible := true;
         aIm_miss[4].Visible := false;
         aIm_miss[7].Visible := true;

         Vid_Show_All_Charlie(false);
       end;
    7:   aIm_miss[8].Visible := true;
    8: begin
         aIm_miss[6].Visible := false;
         aIm_miss[7].Visible := false;
         aIm_miss[8].Visible := false;
       end;
    9: begin
         aIm_miss[2].Visible := false;
         aIm_miss[3].Visible := true;
         aIm_miss[6].Visible := true;
       end;
   10:   aIm_miss[7].Visible := true;
   11: begin
         aIm_miss[8].Visible := true;
         Timer_Miss.Interval := 500;
       end;
   13: if iMiss < 3 then
         // game continues
         begin
           Gam_Set_Game_Stage;

           Gam_Set_Difficulty;
           Timer_Game.Enabled := true;

           bControls := true;
           bControlsRacket := true;

           Timer_Miss.Enabled := false;
         end
       else
         // game is over
         begin
           for i := 6 to 8 do aIm_miss[i].Visible := false;

           Snd_Play(pfs_gameover);

           Timer_Miss.Interval := 1200;
         end;
   14: begin
         // sets high scores
         if (iMode = 5) and (iScore > iHiScoreA) then
           begin
             iHiScoreA := iScore;
             iReg.WriteInteger('', 'HiScore1', iHiScoreA);
           end;

         if (iMode = 7) and (iScore > iHiScoreB) then
           begin
             iHiScoreB := iScore;
             iReg.WriteInteger('', 'HiScore2', iHiScoreB);
           end;

         // snoring Snoopy intermission starts
         iLoopSnore := 1;
         Timer_Snore.Interval := 1;
         Timer_Snore.Enabled := true;

         // sets game mode to "game over"
         iMode := 8; Gam_Set_Mode;

         Timer_Game_Over.Interval := 300000;
         Timer_Game_Over.Enabled := true;

         Timer_Miss.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopMiss);
end;





procedure TForm1.Timer_Miss_SoundsTimer(Sender: TObject);
{ stops playing sound effect during "miss" intermission                        }
begin
  Snd_Silence;

  Timer_Miss_Sounds.Enabled := false;
end;





procedure TForm1.Timer_RacketTimer(Sender: TObject);
{ racket animation                                                             }
begin
  if aIm_snoopy[iSnoopyPosition, 3].Visible then
    begin
      aIm_snoopy[iSnoopyPosition, 3].Visible := false;
      aIm_snoopy[iSnoopyPosition, 4].Visible := true;

      bControls := true;
      bControlsRacket := true;

      Timer_Racket.Enabled := false;
    end;

  if aIm_snoopy[iSnoopyPosition, 2].Visible then
    begin
      aIm_snoopy[iSnoopyPosition, 2].Visible := false;
      aIm_snoopy[iSnoopyPosition, 3].Visible := true;
    end;
end;





procedure TForm1.Timer_Score_BlinkTimer(Sender: TObject);
{ score blinks during the "chance time" intermission                           }
begin
  Vid_Show_Panel_Score(iScore, Odd(iLoopScoreBlink));

  Inc(iLoopScoreBlink);

  if iLoopScoreBlink >= 63 then                                                 // that's (63 * 800 msec =) 50 seconds + random bonus time
    // chance-time ends
    begin
      Vid_Show_Panel_Score(iScore, true);

      iScoreBallCharlie := 2;
      iScoreBallLucy    := 3;

      bChance := false;

      Timer_Score_Blink.Enabled := false;
    end;
end;





procedure TForm1.Timer_SnoreTimer(Sender: TObject);
{ Snoopy snores on his dog house                                               }
  var
    i: integer;
begin
  case iLoopSnore of
    1: aIm_miss[6].Visible := true;
    2: aIm_miss[7].Visible := true;
    3: aIm_miss[8].Visible := true;
    4: for i := 6 to 8 do
         aIm_miss[i].Visible := false;
  end; // case..of

  Timer_Snore.Interval := 550;

  // loops action
  Inc(iLoopSnore);

  if iLoopSnore = 5 then
    iLoopSnore := 1;
end;





{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.pm_0_menuPopup(Sender: TObject);
{ shows the "checked" menuitems if needed. also, hides the game menu if it is  }
{ visible when the popup menu pops up                                          }
begin
{}pm_11_splashscreen.Checked := iReg.ReadBool('', 'SplashScreen', SplashForm.chb_splashscreen.Checked);

{}if bFullscreenMode
    and bGameMenuOpen then
    begin
      bGameMenuOpen := false;
      Form1.Vid_Display_Game_Menu(bGameMenuOpen);
    end;
end;





procedure TForm1.pm_1_aboutClick(Sender: TObject);
{ displays the embedded "About the author..." form                             }
begin
  AboutForm.Show;
end;





procedure TForm1.pm_2_readmeClick(Sender: TObject);
{ opens the "Readme.txt" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Readme.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_3_whatsnewClick(Sender: TObject);
{ opens the "Whatsnew.txt" file                                                }
begin
  ShellExecute(Handle, 'open', 'Whatsnew.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_4_manualClick(Sender: TObject);
{ displays the embedded instructions manual                                    }
begin
  ManualForm.Show;
end;





procedure TForm1.pm_6_configClick(Sender: TObject);
{ opens the "Config.ini" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Config.ini', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_7_keysClick(Sender: TObject);
{ displays the "key settings" form                                             }
begin
  KeysForm.position := poScreenCenter;
  KeysForm.visible := true;
  Form1.Enabled := false;
end;





procedure TForm1.pm_8_hiscoreClick(Sender: TObject);
{ resets high scores to the system registry                                    }
begin
  iHiScoreA := 0; iReg.WriteInteger('', 'HiScore1', iHiScoreA);
  iHiScoreB := 0; iReg.WriteInteger('', 'HiScore2', iHiScoreB);
  Beep;
end;





{}procedure TForm1.pm_10_fullscreenClick(Sender: TObject);
{ changes from fullscreen to windowed, and vice-versa                          }
begin
  // updates popup menu
  bFullscreenMode := not bFullscreenMode;
  pm_10_fullscreen.Checked := bFullscreenMode;
  bGameMenuOpen := false;

  iReg.WriteBool('', 'FullscreenMode', bFullscreenMode);

  // graphics init
  Top  := - Screen.Height;
  Left := - Screen.Width;
  Gfx_Game_Init;
  Position := poScreenCenter;
end;





procedure TForm1.pm_11_splashscreenClick(Sender: TObject);
{ updates the ticks to the checkbox, the popup menu and the system registry    }
begin
{}pm_11_splashscreen.Checked := not pm_11_splashscreen.Checked;

{}iReg.WriteBool('', 'SplashScreen', pm_11_splashscreen.Checked);
end;





procedure TForm1.pm_14_exitClick(Sender: TObject);
{ exits the game                                                               }
begin
  Application.Terminate;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.btn_game_menu_topClick(Sender: TObject);
{ sets the game menu as visible/hidden, then displays/hides it                 }
begin
  bGameMenuOpen := not bGameMenuOpen;
  Vid_Display_Game_Menu(bGameMenuOpen);
end;





procedure TForm1.la_menu_itemMouseEnter(Sender: TObject);
{ the game menu label bg colour becomes solid                                  }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := false;
end;





procedure TForm1.la_menu_itemMouseLeave(Sender: TObject);
{ the game menu label bg colour becomes transparent                            }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := true;
end;





procedure TForm1.la_menu_itemMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ hides the game menu, once a label has been pressed                           }
begin
  if Button = mbLeft then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(false);
    end;
end;





procedure TForm1.Vid_Display_Game_Menu(b: boolean);
{ displays/hides the game menu, depending on the value of bGameMenuOpen        }
  var
    i: integer;
begin
  case bGameMenuOpen of
    true: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i] := TLabel.Create(Form1);
            aMenuLabels[i].Parent := Form1;

            aMenuLabels[i].Top := 32 * i;
            aMenuLabels[i].Left := 0;

            aMenuLabels[i].AutoSize := false;
            aMenuLabels[i].Height := 32;
            aMenuLabels[i].Width := 256;
            aMenuLabels[i].Layout := tlCenter;

            aMenuLabels[i].Transparent := true;

            if aMenuItem[i].Caption <> '-' then
              begin
                aMenuLabels[i].Font.Name := 'Arial';
                aMenuLabels[i].Font.Color := clWhite;
                aMenuLabels[i].Font.Height := 16;
                aMenuLabels[i].Caption := '   ' + aMenuItem[i].Caption;

                if aMenuItem[i].Checked then
                  aMenuLabels[i].Caption := aMenuLabels[i].Caption + ' [V]';

                aMenuLabels[i].Color := $00000000;
                aMenuLabels[i].Cursor := crHandPoint;
                aMenuLabels[i].Enabled := aMenuItem[i].Enabled;

                if aMenuLabels[i].Enabled then
                  begin
                    aMenuLabels[i].OnClick      := aMenuItem[i].OnClick;
                    aMenuLabels[i].OnMouseEnter := la_menu_itemMouseEnter;
                    aMenuLabels[i].OnMouseLeave := la_menu_itemMouseLeave;
                    aMenuLabels[i].OnMouseUp    := la_menu_itemMouseUp;
                  end;
              end;
          end;

        btn_game_menu_background.Width := 256;
        btn_game_menu_background.Height := 32 * (MENU_ITEMS + 1)
      end;
    false: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i].Free;
          end;

        btn_game_menu_background.Height := 32;
        btn_game_menu_background.Width := 36;
      end;
    end; // case..of
end;





initialization
  bStartup := true;





end.
